"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const LIST_TYPE_VIEW = "VIEW";
const LIST_TYPE_TABLE = "BASE TABLE";
class ModelBuilder {
    constructor(knex, databaseName) {
        this.knex = knex;
        this.databaseName = databaseName;
    }
    /**
     * Return a copy of an object but with all keys lower case
     */
    static keysToLower(obj) {
        let newobj = {};
        Object.keys(obj).forEach(key => newobj[key.toLowerCase()] = obj[key]);
        return newobj;
    }
    /**
     * Return the length of a type, eg varchar(255)=>255
     * @param type
     */
    static getMysqlLength(strType) {
        let strip = strType.replace(/\D/g, '');
        return strip.length == 0 ? 0 : parseInt(strip);
    }
    /**
     * Return the length of a type, eg varchar(255)=>varchar
     * @param type
     */
    static stripMysqlLength(type) {
        let pos = type.indexOf("(");
        if (pos > -1) {
            return type.substr(0, pos);
        }
        return type;
    }
    /**
     * return a select query to list all tables or views from a database
     */
    listFromDatabase(listType) {
        if (listType != "BASE TABLE" && listType != "VIEW") {
            throw new Error("Illegal listtype");
        }
        let select = "`information_schema`.`TABLES`.`TABLE_NAME` AS `tname`";
        let from = "`information_schema`.`TABLES`";
        let dbClause = "`information_schema`.`TABLES`.`TABLE_SCHEMA` = '" + this.databaseName + "'";
        let baseTable = "`information_schema`.`TABLES`.`TABLE_TYPE` = '" + listType + "'";
        return `SELECT ${select} FROM ${from} WHERE ${dbClause} AND ${baseTable} `;
    }
    listViews() {
        return __awaiter(this, void 0, void 0, function* () {
            let rows = yield this.knex.raw(this.listFromDatabase(LIST_TYPE_VIEW));
            return rows[0].map(item => item.tname);
        });
    }
    /**
     * Lists all the tables in current database
     */
    listTables() {
        return __awaiter(this, void 0, void 0, function* () {
            let rows = yield this.knex.raw(this.listFromDatabase(LIST_TYPE_TABLE));
            return rows[0].map(item => item.tname);
        });
    }
    columnArrayToDatabaseSchema(colArrMap) {
        var schema = {};
        for (var tableName in colArrMap) {
            colArrMap[tableName] = colArrMap[tableName].map((col, i) => {
                col = ModelBuilder.keysToLower(col);
                col.length = ModelBuilder.getMysqlLength(col.type);
                col.isPrimary = col.key == "PRI";
                col.index = i;
                col.type = ModelBuilder.stripMysqlLength(col.type);
                return col;
            });
            var newTable = {};
            colArrMap[tableName].forEach(col => newTable[col.field] = col);
            schema[tableName] = newTable;
        }
        return schema;
    }
    /**
     * List all columns for a table given table name
     */
    listColumns(tableName) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.knex.raw("SHOW COLUMNS FROM " + tableName).then(colData => colData[0]);
        });
    }
    renderModel(tables) {
        return __awaiter(this, void 0, void 0, function* () {
            //TODO list all in one query instead of one query per table
            var columnArrayMap = {};
            var promises = tables.map((tableName) => __awaiter(this, void 0, void 0, function* () {
                columnArrayMap[tableName] = yield this.listColumns(tableName);
            }));
            yield Promise.all(promises);
            return this.columnArrayToDatabaseSchema(columnArrayMap);
        });
    }
    renderDatabaseSchema() {
        return __awaiter(this, void 0, void 0, function* () {
            let schema = {
                tables: yield this.renderTableModel(),
                views: yield this.renderViewModel()
            };
            return schema;
        });
    }
    renderViewModel() {
        return __awaiter(this, void 0, void 0, function* () {
            var tables = yield this.listViews();
            return yield this.renderModel(tables);
        });
    }
    renderTableModel() {
        return __awaiter(this, void 0, void 0, function* () {
            var tables = yield this.listTables();
            return this.renderModel(tables);
        });
    }
}
exports.default = ModelBuilder;
//# sourceMappingURL=model-builder.js.map